///////////////////////////////////////////////////////////////////////////////
// Name:		wxAnalogSaat.cpp
// Purpose:     Analog clock
// Author:      emarti, Murat zdemir  e-mail: dtemarti<add>gmail<dot>com
// Created:     15.04.2006
// Copyright:   (c) emarti
// Licence:     wxWindows license
///////////////////////////////////////////////////////////////////////////////

#include "wx/wxprec.h" 

#ifdef __BORLANDC__ 
    #pragma hdrstop 
#endif 

#ifndef WX_PRECOMP 
    #include "wx/wx.h" 
#endif 

#include "wxAnalogSaat.h"

BEGIN_EVENT_TABLE(wxAnalogSaat, wxPanel) 
    EVT_PAINT(wxAnalogSaat::OnPaint)
	EVT_TIMER(ID_WXTIMER, wxAnalogSaat::Timer)	
END_EVENT_TABLE()

wxAnalogSaat::wxAnalogSaat(wxWindow *parent, wxWindowID id, int x, int y,int handwidth, const wxImage* clockpic) 
                  : wxPanel(parent, id,  wxPoint(x, y), wxDefaultSize, wxBORDER_NONE) 
{ 
	start = true;
	m_bitmap =  new wxBitmap(*clockpic);
	clockpicwidth = m_bitmap->GetWidth(); 
	SetSize(wxSize(clockpicwidth, clockpicwidth));
	hw = handwidth;
	
	WxTimer = new wxTimer();
	WxTimer->SetOwner(this, ID_WXTIMER);
	WxTimer->Start(TIMER);
} 

wxAnalogSaat::~wxAnalogSaat() 
{ 
	delete WxTimer;
	if(m_bitmap) 
		{ 
			delete m_bitmap;
			m_bitmap = (wxBitmap *) NULL;  
		}	
} 

int modulal(int val)
{
	do{ 
		val +=360;
	}while(val<0);
	return val;
}

int kactane(int x, int y)
{
	int times=0;
	do{ 
		x-=y;
		times++;
	}while(x>=y);
	return times;
}

double sm_angle(int m)
{
	// Values of m must be minute or second. 
	return ((modulal((90 - (m * 6))) * M_PI) / 180);
}

double h_angle(int m, int n)
{
	// Values of m must be hour, values of n must be minute.
	return ((modulal((90 - (m * 30) - (kactane(n, 6) * 3))) * M_PI) / 180);
}

void wxAnalogSaat::OnPaint(wxPaintEvent& WXUNUSED(event)) 
{    
    if(m_bitmap && m_bitmap->Ok()) 
    { 
		wxPaintDC dc(this);
		dc.DrawBitmap(wxBitmap(*m_bitmap),0,0,true); 
		double c = clockpicwidth / 2;
		wxPen pen(*wxWHITE,hw,wxSOLID);
		// clear pre-now
		dc.SetPen(pen);
		dc.DrawLine(c, c, (int)(c + 0.8 * c * cos(xpre)), (int)(c - 0.8 * c * sin(xpre))); // pre-second hand
		// draw now
		pen.SetColour(*wxBLACK);
		pen.SetWidth(hw*3);
		dc.SetPen(pen);
		dc.DrawLine(c, c, (int)(c + 0.65 * c * cos(z)), (int)(c - 0.65  * c * sin(z))); // hour hand
		pen.SetWidth(hw*2);
		dc.SetPen(pen);
		dc.DrawLine(c, c, (int)(c + 0.8 * c * cos(y)), (int)(c - 0.8  * c * sin(y))); // minute hand
		pen.SetWidth(hw);
		dc.SetPen(pen);
		dc.DrawLine(c, c, (int)(c + 0.8 * c * cos(x)), (int)(c - 0.8 * c * sin(x))); // second hand
    } 
} 

void wxAnalogSaat::Timer(wxTimerEvent& WXUNUSED(event))
{
	wxDateTime zmn = wxDateTime::Now();
	if ((zmn.GetSecond() == 0) || start) Refresh();
	start = false;
	
	x = sm_angle(zmn.GetSecond());
	xpre = sm_angle(zmn.GetSecond()-1);
	y = sm_angle(zmn.GetMinute());
	z = h_angle(zmn.GetHour(),zmn.GetMinute());
	Refresh(false); 
}
